@@ -76,14 +76,14 @@ pub mod vec {
     struct Push<T> {
         value: T,
     }
-    impl<T> UpdateTables<Vec<T>, ()> for Push<T>
+    impl<'a, T> UpdateTables<'a, Vec<T>, ()> for Push<T>
     where
         T: Clone,
     {
-        fn apply_first(&mut self, table: &mut Vec<T>) {
+        fn apply_first(&mut self, table: &'a mut Vec<T>) {
             table.push(self.value.clone());
         }
-        fn apply_second(self: Box<Self>, table: &mut Vec<T>) {
+        fn apply_second(self, table: &mut Vec<T>) {
             table.push(self.value); // Move the value instead of cloning.
         }
     }
@@ -92,75 +92,101 @@ pub mod vec {
         index: usize,
         element: T,
     }
-    impl<T> UpdateTables<Vec<T>, ()> for Insert<T>
+    impl<'a, T> UpdateTables<'a, Vec<T>, ()> for Insert<T>
     where
         T: Clone,
     {
-        fn apply_first(&mut self, table: &mut Vec<T>) {
+        fn apply_first(&mut self, table: &'a mut Vec<T>) {
             table.insert(self.index, self.element.clone())
         }
-        fn apply_second(self: Box<Self>, table: &mut Vec<T>) {
+        fn apply_second(self, table: &mut Vec<T>) {
             // Move the value instead of cloning.
             table.insert(self.index, self.element)
         }
     }
 
     struct Clear {}
-    impl<T> UpdateTables<Vec<T>, ()> for Clear {
+    impl<'a, T> UpdateTables<'a, Vec<T>, ()> for Clear {
         fn apply_first(&mut self, table: &mut Vec<T>) {
             table.clear()
         }
+        fn apply_second(mut self, table: &mut Vec<T>) {
+            self.apply_first(table);
+        }
     }
 
     struct Pop {}
-    impl<T> UpdateTables<Vec<T>, Option<T>> for Pop {
-        fn apply_first(&mut self, table: &mut Vec<T>) -> Option<T> {
+    impl<'a, T> UpdateTables<'a, Vec<T>, Option<T>> for Pop {
+        fn apply_first(&mut self, table: &'a mut Vec<T>) -> Option<T> {
             table.pop()
         }
+        fn apply_second(mut self, table: &mut Vec<T>) {
+            self.apply_first(table);
+        }
     }
 
     struct Reserve {
         additional: usize,
     }
-    impl<T> UpdateTables<Vec<T>, ()> for Reserve {
-        fn apply_first(&mut self, table: &mut Vec<T>) {
+    impl<'a, T> UpdateTables<'a, Vec<T>, ()> for Reserve {
+        fn apply_first(&mut self, table: &'a mut Vec<T>) {
             table.reserve(self.additional)
         }
+        fn apply_second(mut self, table: &mut Vec<T>) {
+            self.apply_first(table);
+        }
     }
 
     struct ReserveExact {
         additional: usize,
     }
-    impl<T> UpdateTables<Vec<T>, ()> for ReserveExact {
-        fn apply_first(&mut self, table: &mut Vec<T>) {
+    impl<'a, T> UpdateTables<'a, Vec<T>, ()> for ReserveExact {
+        fn apply_first(&mut self, table: &'a mut Vec<T>) {
             table.reserve(self.additional)
         }
+        fn apply_second(mut self, table: &mut Vec<T>) {
+            self.apply_first(table);
+        }
     }
 
     struct ShrinkToFit {}
-    impl<T> UpdateTables<Vec<T>, ()> for ShrinkToFit {
-        fn apply_first(&mut self, table: &mut Vec<T>) {
+    impl<'a, T> UpdateTables<'a, Vec<T>, ()> for ShrinkToFit {
+        fn apply_first(&mut self, table: &'a mut Vec<T>) {
             table.shrink_to_fit()
         }
+        fn apply_second(mut self, table: &mut Vec<T>) {
+            self.apply_first(table);
+        }
     }
 
     struct Truncate {
         len: usize,
     }
-    impl<T> UpdateTables<Vec<T>, ()> for Truncate {
-        fn apply_first(&mut self, table: &mut Vec<T>) {
+    impl<'a, T> UpdateTables<'a, Vec<T>, ()> for Truncate {
+        fn apply_first(&mut self, table: &'a mut Vec<T>) {
             table.truncate(self.len)
         }
+        fn apply_second(mut self, table: &mut Vec<T>) {
+            self.apply_first(table);
+        }
     }
 
     struct SwapRemove {
         index: usize,
     }
-    impl<T> UpdateTables<Vec<T>, T> for SwapRemove {
-        fn apply_first(&mut self, table: &mut Vec<T>) -> T {
+    impl SwapRemove {
+        fn apply<'a, T>(&mut self, table: &'a mut Vec<T>) -> T {
             table.swap_remove(self.index)
         }
     }
+    impl<'a, T> UpdateTables<'a, Vec<T>, T> for SwapRemove {
+        fn apply_first(&mut self, table: &'a mut Vec<T>) -> T {
+            self.apply(table)
+        }
+        fn apply_second(mut self, table: &mut Vec<T>) {
+            self.apply(table);
+        }
+    }
 
     struct Drain<R>
     where
@@ -168,14 +194,14 @@ pub mod vec {
     {
         range: R,
     }
-    impl<T, R> UpdateTables<Vec<T>, Vec<T>> for Drain<R>
+    impl<'a, T, R> UpdateTables<'a, Vec<T>, std::vec::Drain<'a, T>> for Drain<R>
     where
         R: 'static + Clone + RangeBounds<usize>,
     {
-        fn apply_first(&mut self, table: &mut Vec<T>) -> Vec<T> {
-            table.drain(self.range.clone()).collect()
+        fn apply_first(&mut self, table: &'a mut Vec<T>) -> std::vec::Drain<'a, T> {
+            table.drain(self.range.clone())
         }
-        fn apply_second(self: Box<Self>, table: &mut Vec<T>) {
+        fn apply_second(self, table: &mut Vec<T>) {
             table.drain(self.range);
         }
     }
@@ -187,28 +213,28 @@ pub mod vec {
         f: F,
         _compile_t: std::marker::PhantomData<T>,
     }
-    impl<T, F> UpdateTables<Vec<T>, ()> for Retain<T, F>
+    impl<'a, T, F> UpdateTables<'a, Vec<T>, ()> for Retain<T, F>
     where
         F: 'static + Clone + FnMut(&T) -> bool,
     {
-        fn apply_first(&mut self, table: &mut Vec<T>) {
+        fn apply_first(&mut self, table: &'a mut Vec<T>) {
             table.retain(self.f.clone())
         }
-        fn apply_second(self: Box<Self>, table: &mut Vec<T>) {
+        fn apply_second(self, table: &mut Vec<T>) {
             table.retain(self.f)
         }
     }
 
     // Here we reimplement the mutable interface of a Vec.
-    impl<'w, T> WriteGuard<'w, T>
+    impl<'w, 'a, T> WriteGuard<'w, T>
     where
         T: 'static + Clone + Send,
     {
-        pub fn push(&mut self, value: T) {
+        pub fn push(&'a mut self, value: T) {
             self.guard.update_tables(Push { value })
         }
 
-        pub fn insert(&mut self, index: usize, element: T) {
+        pub fn insert(&'a mut self, index: usize, element: T) {
             self.guard.update_tables(Insert { index, element })
         }
     }
@@ -240,14 +266,10 @@ pub mod vec {
         pub fn swap_remove(&mut self, index: usize) -> T {
             self.guard.update_tables(SwapRemove { index })
         }
+    }
 
-        /// Performs the same mutation on the data as Vec::drain, but instead of
-        /// returning an iterator to the drained elements, it returns a Vec of
-        /// the drained elements.
-        ///
-        /// This is because the return value from update_tables must own its own
-        /// data.
-        pub fn drain<R>(&mut self, range: R) -> Vec<T>
+    impl<'w, 'a, T> WriteGuard<'w, T> {
+        pub fn drain<R>(&'a mut self, range: R) -> std::vec::Drain<'a, T>
         where
             R: 'static + Clone + Send + RangeBounds<usize>,
         {
@@ -485,11 +507,35 @@ mod test {
             for i in 0..5 {
                 wg.push(i + 1);
             }
-            assert_eq!(wg.drain(1..4), vec![2, 3, 4]);
+            assert_eq!(wg.drain(1..4).collect::<Vec<_>>(), vec![2, 3, 4]);
         }
 
         assert_eq!(*reader.read(), vec![1, 5]);
         assert_eq!(*writer.write(), vec![1, 5]);
         assert_eq!(*reader.read(), vec![1, 5]);
     }
+
+    #[test]
+    fn lifetimes() {
+        let mut writer = Writer::<i32>::new();
+        let mut reader = writer.new_reader();
+
+        {
+            let mut wg = writer.write();
+            for i in 0..5 {
+                wg.push(i + 1);
+            }
+            // Switching the order of 'drain' and 'swapped' fails to compile due
+            // to the borrow checker, since 'drain' is tied to the lifetime of
+            // table.
+            let swapped = wg.swap_remove(1);
+            let drain = wg.drain(1..4);
+            assert_eq!(drain.collect::<Vec<_>>(), vec![5, 3, 4]);
+            assert_eq!(swapped, 2);
+        }
+
+        assert_eq!(*reader.read(), vec![1]);
+        assert_eq!(*writer.write(), vec![1]);
+        assert_eq!(*reader.read(), vec![1]);
+    }
 }
