@@ -5,16 +5,84 @@ use slab::Slab;
 use std::fmt;
 
 /// Operations that update the data held internally. Users mutate the tables by
-/// implementing this trait for each function to be performed on the tables.
+/// implementing this trait for each function to be performed on the tables. For
+/// examples check the implementations for the collections.
 ///
 /// Users must be careful to guarantee that apply_first and apply_second cause
-/// the tables to end up in the same state.
-pub trait UpdateTables<T, R> {
-    fn apply_first(&mut self, table: &mut T) -> R;
+/// the tables to end up in the same state. They also must be certain not to use
+/// the return value to mutate the underlying table, since this likely can't be
+/// mimiced in 'apply_second'.
+///
+/// It is *highly* discouraged to create updates which return mutable references
+/// to the table's internals. E.g:
+///
+///```compile_fail
+/// struct MutableRef {}
+/// impl<'a, T> UpdateTables<'a, Vec<T>, &'a mut T> for MutableRef {
+///    fn apply_first(&mut self, table: &'a mut Vec<T>) -> &'a mut T {
+///        &mut table[0]
+///    }
+///    fn apply_second(self, table: &mut Vec<T>) {
+///        &mut table[0];
+///    }
+/// }
+/// ```
+///
+/// Even without the explicit lifetime, which allows for mutable references,
+/// this issue is still possible.
+///
+/// ```
+/// use std::sync::Arc;
+/// use std::cell::RefCell;
+///
+/// fn ret_owned_value<T: Clone>(opt : &Vec<T>) -> T {
+///     opt[0].clone()
+/// }
+///
+/// fn main() {
+///     let opt = vec![Arc::new(RefCell::new(3)), Arc::new(RefCell::new(5))];
+///     let opt_ref = ret_owned_value(&opt);
+///     *opt_ref.borrow_mut() += 1;
+///     println!("{:?}, {:?}", opt_ref, opt);
+///     // prints: "RefCell { value: 4 }, [RefCell { value: 4 }, RefCell { value: 5 }]"
+/// }
+/// ```
+///
+/// Therefore it is also highly recommended not to include types that allow for
+/// interior mutability, since that can lead to the caller returning a reference
+/// to part of an underlying table. If the caller then mutates this outside of
+/// UpdateTables, this is can cause divergence between the tables since
+/// apply_second isn't aware of this mutation.
+///
+/// An example implementation taken from collections::vec:
+/// ```compile_fail
+/// struct SwapRemove {
+///     index: usize,
+/// }
+/// impl SwapRemove {
+///     // Not needed, but can be convenient for making both applies the same.
+///     fn apply<'a, T>(&mut self, table: &'a mut Vec<T>) -> T {
+///         table.swap_remove(self.index)
+///     }
+/// }
+/// impl<'a, T> UpdateTables<'a, Vec<T>, T> for SwapRemove {
+///     fn apply_first(&mut self, table: &'a mut Vec<T>) -> T {
+///         self.apply(table)
+///     }
+///     fn apply_second(mut self, table: &mut Vec<T>) {
+///         self.apply(table);
+///     }
+/// }
+/// ```
+pub trait UpdateTables<'a, T, R> {
+    fn apply_first(&mut self, table: &'a mut T) -> R;
 
-    fn apply_second(mut self: Box<Self>, table: &mut T) {
-        Self::apply_first(&mut self, table);
-    }
+    /// Unfortunately we can't offer a default implementationt to call
+    /// 'apply_first'. This is because we can't constrain 'apply_second' with a
+    /// lifetime on 'table' because this would mean that each update has a
+    /// unique type in 'apply_second', making it impossible for us to hold
+    /// 'ops_to_replay' since each op would have a different type.
+    fn apply_second(self, table: &mut T);
 }
 
 /// Writer is the entry point for using active_standy primitives, and for
@@ -209,11 +277,14 @@ impl<'w, T> WriteGuard<'w, T> {
     /// The update passed in must be valid for 'static because it will outlive
     /// the WriteGuard taking the update, so we can't make any limitations on
     /// it.
-    pub fn update_tables<R>(&mut self, mut update: impl UpdateTables<T, R> + 'static + Sized) -> R {
+    pub fn update_tables<'a, R>(
+        &'a mut self,
+        mut update: impl UpdateTables<'a, T, R> + 'static + Sized,
+    ) -> R {
         let res = update.apply_first(&mut self.table);
 
         self.ops_to_replay.push(Box::new(move |table| {
-            Box::new(update).apply_second(table);
+            update.apply_second(table);
         }));
 
         res
@@ -327,9 +398,9 @@ pub struct SendWriteGuard<'w, T> {
 }
 
 impl<'w, T> SendWriteGuard<'w, T> {
-    pub fn update_tables<R>(
-        &mut self,
-        update: impl UpdateTables<T, R> + 'static + Sized + Send,
+    pub fn update_tables<'a, R>(
+        &'a mut self,
+        update: impl UpdateTables<'a, T, R> + 'static + Sized + Send,
     ) -> R {
         self.guard.update_tables(update)
     }
@@ -350,24 +421,43 @@ mod test {
     struct PushVec<T> {
         value: T,
     }
-    impl<T> UpdateTables<Vec<T>, ()> for PushVec<T>
+    impl<'a, T> UpdateTables<'a, Vec<T>, ()> for PushVec<T>
     where
         T: Clone,
     {
-        fn apply_first(&mut self, table: &mut Vec<T>) {
+        fn apply_first(&mut self, table: &'a mut Vec<T>) {
             table.push(self.value.clone());
         }
-        fn apply_second(self: Box<Self>, table: &mut Vec<T>) {
+        fn apply_second(self, table: &mut Vec<T>) {
             table.push(self.value); // Move the value instead of cloning.
         }
     }
 
     struct PopVec {}
-    impl<T> UpdateTables<Vec<T>, Option<T>> for PopVec {
-        fn apply_first(&mut self, table: &mut Vec<T>) -> Option<T> {
+    impl PopVec {
+        fn apply<'a, T>(&mut self, table: &'a mut Vec<T>) -> Option<T> {
             table.pop()
         }
     }
+    impl<'a, T> UpdateTables<'a, Vec<T>, Option<T>> for PopVec {
+        fn apply_first(&mut self, table: &'a mut Vec<T>) -> Option<T> {
+            self.apply(table)
+        }
+        fn apply_second(mut self, table: &mut Vec<T>) {
+            (&mut self).apply(table);
+        }
+    }
+
+    /// This is an example of what not to do!
+    struct MutableRef {}
+    impl<'a, T> UpdateTables<'a, Vec<T>, &'a mut T> for MutableRef {
+        fn apply_first(&mut self, table: &'a mut Vec<T>) -> &'a mut T {
+            &mut table[0]
+        }
+        fn apply_second(self, table: &mut Vec<T>) {
+            &mut table[0];
+        }
+    }
 
     #[test]
     fn one_write_guard() {
@@ -528,4 +618,20 @@ mod test {
             "ReadGuard { active_table: [2] }"
         );
     }
+
+    #[test]
+    fn mutable_ref() {
+        // Show that when the Writer is dropped, Readers remain valid.
+        let mut writer = Writer::<Vec<i32>>::default();
+        let mut reader = writer.new_reader();
+
+        {
+            let mut wg = writer.write();
+            wg.update_tables(PushVec { value: 2 });
+            let mr = wg.update_tables(MutableRef {});
+            *mr = 10;
+        }
+
+        assert_eq!(*reader.read(), vec![10]);
+    }
 }
